-- B37004B.ADA

--                             Grant of Unlimited Rights
--
--     Under contracts F33600-87-D-0337, F33600-84-D-0280, MDA903-79-C-0687,
--     F08630-91-C-0015, and DCA100-97-D-0025, the U.S. Government obtained 
--     unlimited rights in the software and documentation contained herein.
--     Unlimited rights are defined in DFAR 252.227-7013(a)(19).  By making 
--     this public release, the Government intends to confer upon all 
--     recipients unlimited rights  equal to those held by the Government.  
--     These rights include rights to use, duplicate, release or disclose the 
--     released technical data and computer software in whole or in part, in 
--     any manner and for any purpose whatsoever, and to have or permit others 
--     to do so.
--
--                                    DISCLAIMER
--
--     ALL MATERIALS OR INFORMATION HEREIN RELEASED, MADE AVAILABLE OR
--     DISCLOSED ARE AS IS.  THE GOVERNMENT MAKES NO EXPRESS OR IMPLIED 
--     WARRANTY AS TO ANY MATTER WHATSOEVER, INCLUDING THE CONDITIONS OF THE
--     SOFTWARE, DOCUMENTATION OR OTHER INFORMATION RELEASED, MADE AVAILABLE 
--     OR DISCLOSED, OR THE OWNERSHIP, MERCHANTABILITY, OR FITNESS FOR A
--     PARTICULAR PURPOSE OF SAID MATERIAL.
--*
-- CHECK THAT A DEFAULT RECORD COMPONENT INITIALIZATION
-- CANNOT BE SPECIFIED FOR A COMPONENT THAT HAS TYPE
-- TASK, LIMITED PRIVATE, OR COMPOSITE CONTAINING EITHER.

-- DAT 6/18/81
-- JRK 6/6/84

PROCEDURE B37004B IS

     PACKAGE PK IS

          TYPE L IS LIMITED PRIVATE;
          TASK TYPE T;

          TYPE A IS ARRAY(1..1) OF L;
          TYPE B IS ARRAY(1..1) OF T;

          TYPE R1 IS RECORD
               C : L;
          END RECORD;

          TYPE R2 IS RECORD
               C : T;
          END RECORD;

          CL : CONSTANT L;
          TSK : T;

          FUNCTION FA RETURN A;
          FUNCTION FB RETURN B;
          FUNCTION FR1 RETURN R1;
          FUNCTION FR2 RETURN R2;

          TYPE R3 IS RECORD
               C1 : L := CL;            -- ERROR: NO := FOR L.
               C2 : T := TSK;           -- ERROR: NO := FOR T.
               C3 : A := FA;            -- ERROR: NO := FOR A.
               C4 : B := FB;            -- ERROR: NO := FOR B.
               C5 : R1 := FR1;          -- ERROR: NO := FOR R1.
               C6 : R2 := FR2;          -- ERROR: NO := FOR R2.
               C7 : B;                  -- OK.
          END RECORD;

     PRIVATE

          TYPE L IS (E);
          CL : CONSTANT L := E;

          TYPE R4 IS RECORD
               C1 : L := CL;            -- OK.
               C2 : T := TSK;           -- ERROR: NO := FOR T.
               C3 : A := FA;            -- OK.
               C4 : B := FB;            -- ERROR: NO := FOR B.
               C5 : R1 := FR1;          -- OK.
               C6 : R2 := FR2;          -- ERROR: NO := FOR R2.
          END RECORD;

     END PK; 

     USE PK;

     TYPE R5 IS RECORD
          C1 : L := CL;                 -- ERROR: NO := FOR L.
          C2 : T := TSK;                -- ERROR: NO := FOR T.
          C3 : A := FA;                 -- ERROR: NO := FOR A.
          C4 : B := FB;                 -- ERROR: NO := FOR B.
          C5 : R1 := FR1;               -- ERROR: NO := FOR R1.
          C6 : R2 := FR2;               -- ERROR: NO := FOR R2.
     END RECORD;

     O1 : L;                            -- OK.
     O2 : T;                            -- OK.
     O3 : A;                            -- OK.
     O4 : B;                            -- OK.
     O5 : R1;                           -- OK.
     O6 : R2;                           -- OK.

     PACKAGE BODY PK IS

          TYPE R6 IS RECORD
               C1 : L := CL;            -- OK.
               C2 : T := TSK;           -- ERROR: NO := FOR T.
               C3 : A := FA;            -- OK.
               C4 : B := FB;            -- ERROR: NO := FOR B.
               C5 : R1 := FR1;          -- OK.
               C6 : R2 := FR2;          -- ERROR: NO := FOR R2.
          END RECORD;

          TASK BODY T IS
          BEGIN
               NULL;
          END T;

          FUNCTION FA RETURN A IS
          BEGIN
               RETURN O3;
          END FA;

          FUNCTION FB RETURN B IS
          BEGIN
               RETURN O4;
          END FB;

          FUNCTION FR1 RETURN R1 IS
          BEGIN
               RETURN O5;
          END FR1;

          FUNCTION FR2 RETURN R2 IS
          BEGIN
               RETURN O6;
          END FR2;

     END PK;

BEGIN
     NULL;
END B37004B;
